import config from "./config.json";
import fetch from "node-fetch";
import FeedParser from "feedparser";
import parser from "fast-xml-parser";
import { NDIProfile } from "./types";
import events from "events";

/* ALGORITHM
  Receive url, name and fps
  Start a coder job for the stream
  Store edit url for job
*/

const parseOptions: Partial<parser.X2jOptions> = {
  attributeNamePrefix: "",
  ignoreAttributes: false,
  ignoreNameSpace: true,
};

type Resolution = {
  width: number;
  height: number;
  fps: number;
};

export default class CoderHandler {
  private address: URL;
  private streamName: string;
  private fps: number;
  private link?: Promise<string>;
  private profile: NDIProfile;
  //private error: string = "";

  private StreamLog: events.EventEmitter.EventEmitter | undefined;

  constructor(
    address: URL,
    streamName: string,
    profile: NDIProfile,
    streamLog: events.EventEmitter.EventEmitter,
    fps: number = 30
  ) {
    this.address = address;
    this.streamName = streamName;
    this.fps = fps;
    this.profile = profile;

    if (config.logCoderEvents) {
      this.StreamLog = streamLog;
    }
  }

  public async start() {
    this.StreamLog?.emit("event", "log", "coder job started");

    await this.link
      ?.then((link) => fetch(link, { method: "DELETE" }))
      .catch((err) =>
        this.StreamLog?.emit("event", "warning", "Someone else killed the cat.")
      );

    this.link = fetch(
      `http://${config.coder.hostname}:${config.coder.port}/jobs/`,
      this.createVCOS()
    )
      .then((res) => res.text())
      .then((xmlStr) => this.findLink(xmlStr, "self"));

    this.StreamLog?.emit("event", "Log", "CoderLink: " + (await this.link));

    this.link?.then((linkstr) => {
      setTimeout(() => this.printError(linkstr), 2000); //Wait for Coder to try starting job
    });
  }

  private printError(link: string) {
    fetch(link)
      .then((response) => response.text())
      .then((xmlStr) => {
        const obj = parser.parse(xmlStr, parseOptions);
        return obj?.entry?.error?.description || "no error";
      })
      .then((errStr) => this.StreamLog?.emit("event", "Log", errStr));
  }

  public async update(streamName: string, fps: number = 30) {
    this.stop();
    this.streamName = streamName;
    this.fps = fps;
    this.start();
  }

  private findLink(xmlStr: string, rel: string): string {
    const obj = parser.parse(xmlStr, parseOptions);
    return obj.entry.link.find((link: any) => link.rel === rel).href;
  }

  private async getDocument(link: string): Promise<string> {
    return fetch(link)
      .then((res) => res.text())
      .then((xmlStr) => parser.parse(xmlStr, parseOptions));
  }

  public async stop() {
    this.link
      ?.then((link) => fetch(link, { method: "DELETE" }))
      .catch((err) =>
        console.log(
          config.backend.loggingPrefix,
          "Someone else killed the cat.",
          err
        )
      )
      .then(() => {
        this.link = undefined;
      });
  }

  public async delete() {
    this.stop();
  }

  private createVCOS(method: string = "POST") {
    const resolution = this.profile.resolution;
    const height = resolution.height;
    const width = resolution.width;
    const profileName = "transmit_ndi_1080p30"; //this.profile.name;
    const fps = resolution.fps;

    return {
      method,
      headers: { "Content-Type": "application/vnd.vizrt.vcos+xml" },
      body: `<?xml version='1.0' encoding='utf-8'?>
    <vcos xmlns="http://www.vizrt.com/2013/vcos">
        <inputs>
            <input id="Input0" url="${this.address.protocol}//@${this.address.host}" />
        </inputs>
        <outputs>
            <output destination="ndi://key/${this.streamName}" id="Output0" mix="Mix0" profile="${profileName}" />
        </outputs>
        <audiomix>
            <matrix id="Mix0">
                <in_channel>1.0</in_channel>
                <in_channel>1.0</in_channel>
            </matrix>
        </audiomix>
        <profiles>
            <profile id="${profileName}">
            <video>
                    <resize_mode>pad</resize_mode>
                    <aspect>16:9</aspect>
                    <width>${width}</width>
                    <field_order>progressive</field_order>
                    <pixel_format>r8g8b8a8</pixel_format>
                    <height>${height}</height>
                    <fps>${fps}:1</fps>
                </video>
                <audio>
                    <frequency>${this.profile.audiofreq}</frequency>
                    <channels>2</channels>
                </audio>
            </profile>
        </profiles>
    </vcos>`,
    };
  }
}
