import { StreamStatus, ConnectionStatus } from "./types";
import { spawn, ChildProcess } from "child_process";
import events from "events";

import config from "./config.json";

import readline from "readline";

const sltPath = config.backend.SRTLiveTransmit;

export default class SLTHandler {
  private process: ChildProcess | null = null;
  private listener: URL;
  private caller: URL;

  status: StreamStatus = StreamStatus.STOPPED;

  private StreamLog: events.EventEmitter.EventEmitter | undefined;

  StatusChange: events.EventEmitter.EventEmitter = new events.EventEmitter();

  private targetConnected: boolean = false;
  private sourceConnected: boolean = false;
  connectionStatus: ConnectionStatus = ConnectionStatus.NONE;

  constructor(
    listenerURL: URL,
    callerURL: URL,
    streamLog: events.EventEmitter.EventEmitter
  ) {
    this.listener = listenerURL;
    this.caller = callerURL;

    if (config.logSRTEvents) {
      this.StreamLog = streamLog;
    }
  }

  public StartTransmission(): void {
    let slt = spawn(sltPath, [
      "-v",
      this.listener.toString(),
      this.caller.toString(),
    ]);
    this.process = slt;

    this.status = StreamStatus.STARTED;

    //#region StreamListeners

    slt.stdout.on("data", (data) => {
      this.StreamLog?.emit("event", "stdout", data);
    });

    const readInterface = readline.createInterface({
      input: slt.stderr,
    });

    readInterface.on("line", (line) => {
      switch (line) {
        case "SRT target connected":
          this.targetConnected = true;
          break;
        case "SRT target disconnected":
          this.targetConnected = false;
          break;
        case `Accepted SRT source connection`:
          this.sourceConnected = true;
          break;
        case `SRT source disconnected`:
          this.sourceConnected = false;
          break;
        default:
          return;
      }
      if (this.targetConnected && this.sourceConnected) {
        this.connectionStatus = ConnectionStatus.BOTH;
      } else if (this.sourceConnected) {
        this.connectionStatus = ConnectionStatus.SOURCE;
      } else if (this.targetConnected) {
        this.connectionStatus = ConnectionStatus.TARGET;
      } else {
        this.connectionStatus = ConnectionStatus.NONE;
      }
      this.StatusChange.emit("STATUSCHANGE");
    });

    slt.stderr.on("data", (data) => {
      this.StreamLog?.emit("event", "stderr", data);
      /* console.log(config.backend.loggingPrefix + `${this.id}: stderr: ${data}`); */
    });

    slt.on("error", (error) => {
      this.StreamLog?.emit("event", "error", error);

      /* console.log(
        config.backend.loggingPrefix + `${this.id}: error: ${error.message}`
      ); */
      this.status = StreamStatus.CRASHED;
      this.StatusChange.emit("STATUSCHANGE");
    });

    slt.on("close", (code) => {
      this.StreamLog?.emit("event", "close", code);
      /* console.log(
        config.backend.loggingPrefix +
          `${this.id}: child process exited with code ${code}`
      ); */
    });
    //#endregion
  }

  public StopTransmission(): void {
    this.process?.kill("SIGINT");
    this.status = StreamStatus.STOPPED;
    this.connectionStatus = ConnectionStatus.NONE;
  }

  update(listenerURL: URL, callerURL: URL) {
    this.listener = listenerURL;
    this.caller = callerURL;

    if (this.status != StreamStatus.STOPPED) {
      this.StopTransmission();

      this.StartTransmission();
      return "Transmission Restarted with new info,";
    }
    return "";
  }
}
