import CoderHandler from "./CoderHandler";
import { StreamStartInfo, StreamStatus } from "./types";
import SLTHandler from "./SLTHandler";
import { v4 as uuidv4 } from "uuid";
import config from "./config.json";
import { Param, StreamDataForm } from "./types";
import events from "events";

export default class StreamHandler {
  private sltHandler: SLTHandler;
  private coderHandler: CoderHandler;

  id: string;

  listenerURL: URL;
  callerURL: URL;

  private currentStartInfo: StreamStartInfo;

  private name: string;

  private ManagerUpdate: events.EventEmitter.EventEmitter;

  private StreamLogIn: events.EventEmitter.EventEmitter =
    new events.EventEmitter();
  public StreamLogOut: events.EventEmitter.EventEmitter =
    new events.EventEmitter();

  constructor(
    startInfo: StreamStartInfo,
    listenerPort: string,
    callerPort: string,
    ManagerUpdate: events.EventEmitter.EventEmitter
  ) {
    this.ManagerUpdate = ManagerUpdate;
    this.id = uuidv4();

    this.listenerURL = this.createListenerURL(
      listenerPort,
      startInfo.listenerParams
    );

    this.callerURL = this.createCallerURL(callerPort, startInfo.callerParams);

    this.sltHandler = new SLTHandler(
      this.listenerURL,
      this.callerURL,
      this.StreamLogIn
    );

    this.sltHandler.StatusChange.on("STATUSCHANGE", () => {
      ManagerUpdate.emit("event");
    });

    this.coderHandler = new CoderHandler(
      this.callerURL,
      startInfo.name,
      startInfo.ndiProfile,
      this.StreamLogIn
    );

    this.StreamLogIn.on("event", (type, data) => {
      this.StreamLogOut?.emit("event", {
        ID: this.id,
        Name: this.name,
        Type: type,
        Data: data,
      });
    });

    this.name = startInfo.name;

    this.currentStartInfo = startInfo;

    if (startInfo.status == StreamStatus.STARTED) {
      this.StartTransmission();
    }
  }

  public StartTransmission(): void {
    this.sltHandler.StartTransmission();
    this.coderHandler.start();
  }

  public StopTransmission(): void {
    this.sltHandler.StopTransmission();
    this.coderHandler.stop();
  }

  public updateTransmission(startInfo: StreamStartInfo): string {
    //TODO: Coder Update
    let extra = "";

    this.coderHandler.update(startInfo.name);

    //#region SLT Update
    if (
      startInfo.callerParams != this.currentStartInfo.callerParams ||
      startInfo.listenerParams != this.currentStartInfo.listenerParams
    ) {
      this.listenerURL = this.createListenerURL(
        this.listenerURL.port,
        startInfo.listenerParams
      );

      this.callerURL = this.createCallerURL(
        this.callerURL.port,
        startInfo.callerParams
      );

      extra += this.sltHandler.update(this.listenerURL, this.callerURL);
    }
    //#endregion

    //#region StreamHandlerUpdate

    this.name = startInfo.name;

    return "updated " + this.id + extra;
    //#endregion
  }

  private createListenerURL(listenerPort: string, params: Param[]): URL {
    let newURL: URL = this.createURL(
      config.backend.hostname,
      listenerPort,
      params
    );

    //newURL.searchParams.append(`mode`, `listener`);

    return newURL;
  }

  private createCallerURL(callerPort: string, params: Param[]): URL {
    let newURL: URL = this.createURL(config.coder.hostname, callerPort, params);

    //newURL.searchParams.append(`mode`, `caller`);

    return newURL;
  }

  private createURL(hostName: string, port: string, params: Param[]): URL {
    let newURL = new URL(`srt://${hostName}:${port}`);

    params.forEach(([key, value]) => {
      newURL.searchParams.append(key, value);
    });

    return newURL;
  }

  public getStatus() {
    //TODO: Coder Status
    return this.sltHandler.status;
  }

  public toDataForm(): StreamDataForm {
    let connection = new URL(this.listenerURL.toString());
    //connection.searchParams.set("mode", "caller");
    //connection.searchParams.set("latency", "800");
    //connection.searchParams.set("maxbw", "200000");
    //console.log(`printConnection: `, connection);
    connection.searchParams.delete("mode");

    return {
      id: this.id,
      connection: `${connection.protocol}//${config.backend.publicHostName}:${connection.port}${connection.search}`,
      status: this.getStatus(),
      name: this.name,
      listenerParams: this.currentStartInfo.listenerParams,
      callerParams: this.currentStartInfo.callerParams,
      connectionStatus: this.sltHandler.connectionStatus,
      ndiProfile: this.currentStartInfo.ndiProfile,
    };
  }
}
