import { StreamStartInfo, StreamDataForm, StreamStatus } from "./types";
import SLTHandler from "./SLTHandler";
import events from "events";
import config from "./config.json";
import StreamHandler from "./StreamHandler";
import { range } from "./ArrayUtils";
import e from "express";

export default class StreamManager {
  private handlers: Map<string, StreamHandler>;

  public SMLog = new events.EventEmitter();
  public STLMUpdate = new events.EventEmitter();
  public childUpdate = new events.EventEmitter();

  private backendPorts: Set<number>;
  private coderPorts: Set<number>;

  constructor() {
    this.handlers = new Map<string, StreamHandler>();

    this.backendPorts = new Set(
      range(config.backend.streamPorts[0], config.backend.streamPorts[1])
    );
    this.coderPorts = new Set(
      range(config.coder.inputPorts[0], config.coder.inputPorts[1])
    );

    this.childUpdate.on("event", () => {
      this.STLMUpdate?.emit("event", this.getStreamsDataForm());
    });

    const sendInitialUpdate = () => {
      this.STLMUpdate?.emit("event", this.getStreamsDataForm());
      console.log(
        config.backend.loggingPrefix + "Startup Sending Initial update"
      );
    };

    setTimeout(sendInitialUpdate, 5000);
  }

  public createTransmission(startInfo: StreamStartInfo): string {
    //#region select Ports
    if (this.backendPorts.size < 1) {
      let err = "failed: no empty backend listener port.";
      console.log(config.backend.loggingPrefix + err);
      return err;
    }
    let listenerPort: number = this.backendPorts.values().next().value;
    this.backendPorts.delete(listenerPort);

    if (this.coderPorts.size < 1) {
      let err = "failed: no empty coder caller port.";
      console.log(config.backend.loggingPrefix + err);
      return err;
    }
    let callerPort: number = this.coderPorts.values().next().value;
    this.coderPorts.delete(callerPort);

    //#endregion

    let newStreamHandler: StreamHandler = new StreamHandler(
      startInfo,
      listenerPort.toString(),
      callerPort.toString(),
      this.childUpdate
    );

    this.handlers.set(newStreamHandler.id, newStreamHandler);

    newStreamHandler.StreamLogOut.on("event", ({ ID, Name, Type, Data }) => {
      if (Data == null) {
        console.log(config.backend.loggingPrefix, Name, Type, "NULL?");
      } else {
        console.log(config.backend.loggingPrefix, Name, Type, Data.toString());
      }
    });

    this.STLMUpdate?.emit("event", this.getStreamsDataForm());

    let msg = "success: created with id " + newStreamHandler.id;
    console.log(config.backend.loggingPrefix + msg);
    return msg;
  }

  public startStream(streamID: string): string {
    let foundStreamHandler: StreamHandler | undefined =
      this.handlers.get(streamID);

    if (foundStreamHandler != undefined) {
      if (foundStreamHandler.getStatus() != StreamStatus.STARTED) {
        foundStreamHandler.StartTransmission();
        this.STLMUpdate.emit("event", this.getStreamsDataForm());

        let msg = "success: started transmission";
        console.log(config.backend.loggingPrefix + msg);
        return msg;
      } else {
        let err = "failed: Transmission already started.";
        console.log(config.backend.loggingPrefix + err);
        return err;
      }
    } else {
      let err = "failed: Transmission does not exist.";
      console.log(config.backend.loggingPrefix + err);
      return err;
    }
  }

  public stopStream(streamID: string): string {
    let foundStreamHandler: StreamHandler | undefined =
      this.handlers.get(streamID);

    if (foundStreamHandler != undefined) {
      foundStreamHandler.StopTransmission();
      this.STLMUpdate.emit("event", this.getStreamsDataForm());

      let msg = "success: Stopped Transmission";
      console.log(config.backend.loggingPrefix + msg);
      return msg;
    } else {
      let err = "failed: Transmission does not exist.";
      console.log(config.backend.loggingPrefix + err);
      return err;
    }
  }

  public updateTransmission(
    streamID: string,
    startInfo: StreamStartInfo
  ): string {
    let foundStreamHandler: StreamHandler | undefined =
      this.handlers.get(streamID);

    if (foundStreamHandler != undefined) {
      let response: string =
        "success: " + foundStreamHandler.updateTransmission(startInfo);
      this.STLMUpdate.emit("event", this.getStreamsDataForm());
      console.log(config.backend.loggingPrefix + response);
      return response;
    } else {
      let err = "failed: transmission not found.";
      console.log(config.backend.loggingPrefix + err);
      return err;
    }
  }

  public deleteStream(streamID: string): string {
    let foundStreamHandler: StreamHandler | undefined =
      this.handlers.get(streamID);

    if (foundStreamHandler != undefined) {
      foundStreamHandler.StopTransmission();

      this.coderPorts.add(parseInt(foundStreamHandler.callerURL.port));
      this.backendPorts.add(parseInt(foundStreamHandler.listenerURL.port));

      this.handlers.delete(streamID);
      this.STLMUpdate.emit("event", this.getStreamsDataForm());
      let msg = "success: Deleted Transmission";
      console.log(config.backend.loggingPrefix + msg);
      return msg;
    } else {
      let err = "failed: Transmission does not exist.";
      console.log(config.backend.loggingPrefix + err);
      return err;
    }
  }

  public getStream(streamID: string): object | undefined {
    return this.handlers.get(streamID)?.toDataForm();
  }
  public getStreams(): Array<StreamHandler> {
    return Array.from(this.handlers.values());
  }

  public getStreamsDataForm(): StreamDataForm[] {
    return Array.from(this.handlers.values()).map((slth) => slth.toDataForm());
  }
}
