import React, { ChangeEvent, useState } from "react";
import PlayArrowIcon from "@material-ui/icons/PlayArrow";
import PhonelinkSetupIcon from "@material-ui/icons/PhonelinkSetup";
import PauseIcon from "@material-ui/icons/Pause";
import DeleteIcon from "@material-ui/icons/Delete";
import EditIcon from "@material-ui/icons/Edit";
import { IconButton, Tooltip, Typography } from "@material-ui/core";
import Box from "@material-ui/core/Box";
import { TextField } from "@material-ui/core";
import { withTheme } from "@material-ui/core/styles";
import QRCode from "./QRCode";
import { StreamProps } from "./../types";
import FiberManualRecordIcon from "@material-ui/icons/FiberManualRecord";
import {
  deleteStream,
  startStream,
  stopStream,
  updateStream,
} from "./../backend-utils";
import { getGroveURL } from "../grove-utils";
import { ConnectionStatus, StreamStatus } from "../enums";
import { CopyToClipboard } from "react-copy-to-clipboard";

function StreamRow({ stream, setStream }: StreamProps) {
  const [disabled, setDisabled] = useState<boolean>(true);

  //#region Clipboard
  const [copiedConnection, setCopiedConnection] = useState<boolean>(false);

  let onConnectionClickedTimer = false;
  const onConnectionClicked = () => {
    if (onConnectionClickedTimer) return;
    onConnectionClickedTimer = true;
    setCopiedConnection(true);

    setTimeout(function () {
      setCopiedConnection(false);
      onConnectionClickedTimer = false;
    }, 5000);
  };

  const [copiedName, setCopiedName] = useState<boolean>(false);

  let onNameClickedTimer = false;
  const onNameClicked = () => {
    if (onNameClickedTimer) return;
    onNameClickedTimer = true;
    setCopiedName(true);

    setTimeout(function () {
      setCopiedName(false);
      onNameClickedTimer = false;
    }, 5000);
  };
  //#endregion

  //#region ButtonHandlers
  const onPlayButtonClicked = () => {
    if (disabled) {
      startStream(stream.id).then((text) => console.log(text));
    }
  };

  const onPauseButtonClicked = () => {
    stopStream(stream.id).then((text) => console.log(text));
  };

  const onEditButtonClicked = () => {
    if (!disabled) {
      updateStream(stream).then((text) => console.log(text));
    }
    setDisabled(!disabled);
  };

  const onDeleteButtonClicked = () => {
    deleteStream(stream.id).then((text) => console.log(text));
  };

  //#endregion

  const onNameTextFieldChange = (
    event: ChangeEvent<HTMLInputElement | HTMLTextAreaElement>
  ) => {
    const text: string = event.target.value;
    setStream({
      id: stream.id,
      connection: stream.connection,
      status: stream.status,
      name: text,
      listenerParams: stream.listenerParams,
      callerParams: stream.callerParams,
      connectionStatus: stream.connectionStatus,
      ndiProfile: stream.ndiProfile,
    });
  };

  //#region statusChanges
  const getColor: (status: StreamStatus) => string = (status: StreamStatus) => {
    //stopped, started, crashed
    //red,green,yellow
    const colors = ["#FE2222", "#22FE22", "#e39f29"];
    return colors[status];
  };

  const getTooltip: (status: StreamStatus) => string = (
    status: StreamStatus
  ) => {
    //stopped started, crashed
    const toolTip = [
      "Stream is stopped",
      "Stream is running",
      "Stream has crashed",
    ];
    return toolTip[status];
  };

  const getCSColor: (status: ConnectionStatus) => string = (
    status: ConnectionStatus
  ) => {
    //stopped, started, crashed
    //red,green,yellow
    const colors = ["#FE2222", "#e39f29", "#e39f29", "#22FE22"];
    return colors[status];
  };

  const getCSTooltip: (status: ConnectionStatus) => string = (
    status: ConnectionStatus
  ) => {
    //stopped started, crashed
    const toolTip = [
      "All Disconected",
      "Coder Connected",
      "Source Connected",
      "All Connected",
    ];
    return toolTip[status];
  };
  //#endregion

  const getEditText: (disabled: boolean) => string = (disabled: boolean) => {
    //stopped started, crashed
    return disabled ? "Edit Stream" : "Save Stream";
  };

  const boxStyle: Partial<React.CSSProperties> = {
    margin: "10px",
    padding: "5px",
  };

  return (
    <Box
      border={0}
      borderRadius="borderRadius"
      bgcolor="#c07c51"
      className="Stream-Row"
      style={boxStyle}
      onContextMenu={(e: any) => e.preventDefault()}
    >
      <Box>
        <CopyToClipboard text={stream.connection} onCopy={onNameClicked}>
          <TextField
            disabled={disabled}
            color="secondary"
            id="name"
            label="Transmission Name"
            onChange={onNameTextFieldChange}
            value={stream.name}
            variant="filled"
            style={{ marginBottom: "8px", marginRight: "8px", color: "red" }}
            helperText={copiedName ? "Copied to clipboard" : ""}
          />
        </CopyToClipboard>

        <CopyToClipboard text={stream.connection} onCopy={onConnectionClicked}>
          <TextField
            disabled
            color="primary"
            id="connection"
            label="Connection"
            value={stream.connection}
            variant="filled"
            helperText={copiedConnection ? "Copied to clipboard" : ""}
          />
        </CopyToClipboard>
      </Box>

      <Typography variant="subtitle1">{`Profile: ${stream.ndiProfile.description}`}</Typography>

      <Tooltip title={getTooltip(stream.status)}>
        <IconButton aria-label="status">
          <FiberManualRecordIcon
            fontSize="large"
            htmlColor={getColor(stream.status)}
          />
        </IconButton>
      </Tooltip>
      <Tooltip title={getCSTooltip(stream.connectionStatus)}>
        <IconButton aria-label="status">
          <FiberManualRecordIcon
            fontSize="large"
            htmlColor={getCSColor(stream.connectionStatus)}
          />
        </IconButton>
      </Tooltip>

      <Tooltip title="Start Stream">
        <IconButton aria-label="play" onClick={onPlayButtonClicked}>
          <PlayArrowIcon fontSize="large" htmlColor="#25DD25" />
        </IconButton>
      </Tooltip>

      <Tooltip title="Stop Stream">
        <IconButton aria-label="pause" onClick={onPauseButtonClicked}>
          <PauseIcon fontSize="large" color="secondary" />
        </IconButton>
      </Tooltip>

      <Tooltip title={getEditText(disabled)}>
        <IconButton aria-label="edit" onClick={onEditButtonClicked}>
          <EditIcon fontSize="large" color="secondary" />
        </IconButton>
      </Tooltip>

      <Tooltip title="Delete Stream">
        <IconButton aria-label="delete" onClick={onDeleteButtonClicked}>
          <DeleteIcon fontSize="large" color="secondary" />
        </IconButton>
      </Tooltip>

      <Tooltip title="Open Larix" onContextMenu={(e: any) => true}>
        <IconButton aria-label="grove" href={getGroveURL(stream)}>
          <PhonelinkSetupIcon fontSize="large" color="secondary" />
        </IconButton>
      </Tooltip>

      <QRCode text={getGroveURL(stream)} />
    </Box>
  );
}

export default withTheme(StreamRow);
